//
// C++ Implementation: fdatabase
//
// Description: 
//
//
// Author: peter komar <peter@linux-nlyt>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "fdatabase.h"

#include <QSqlQuery>
#include <QSqlError>
#include <QVariant>
#include <QDebug>
#include <QMessageBox>

/*!
    \fn FDataBase::load_data_from_creditors()
 */
QList<Db_dat> load_data_from_creditors()
{
    QList<Db_dat> sRet;
    
    Db_dat data;
    
    if ( !connectDB( "qfinances-db.db" ) )
	    return sRet;
	
    QSqlQuery query;
    query.prepare("select * from creditors where 1");
    query.exec();
    while (query.next())
    {
	    QString date = query.value(0).toString();
	    
	    data.date = date;
		
	    QString sName = query.value(1).toString();
	    data.Name = sName;
		
	    int icount = query.value(2).toInt();
	    data.count_d = icount;
		
	    QString desc = query.value(3).toString();
	    data.descr = desc;
	    
	    sRet.append(data);
    }
    
    return sRet;
}


/*!
    \fn FDataBase::load_data_from_operation()
 */
QString load_data_from_operation(const QString& sY)
{
	QString s_Ret;
		
	
	QString font = "<font size=+1 color=\"blue\"><b>%1</b>:  Count:%2 grn</font><br>";
	
		
	if ( !connectDB( "qfinances-db.db" ) )
		return QString("<font color=\"red\">Error connect DB</font>");
	
	QSqlQuery query;
	query.prepare("select * from bank where 1");
	query.exec();
	int n_C=0;
	while (query.next())
		n_C = query.value(0).toInt();
	
	s_Ret = font.arg(QObject::tr("Operation")).arg(n_C);
	
	s_Ret += "<table border=0 width=100%><tr bgcolor=\"#726f7c\"><th>Data</th><th>Type</th><th>Count</th><th>Description</th></tr>";
	
	query.prepare("select * from operation where 1");
	
	bool bfindet=false;
	
	query.exec();
	while (query.next())
	{
		QString date = query.value(0).toString();
		
		if(date.right(4) == sY)
		{
		   s_Ret += QString("<tr bgcolor=\"#EEF6FF\"><td><font color=\"blue\">%1</font></td>").arg(date);
		
		   QString stype = query.value(1).toString();
		   s_Ret += QString("<td><font color=\"#0F990F\">%1</font></td>").arg(stype);
		
		   int icount = query.value(2).toInt();
		   s_Ret += QString("<td><font color=\"#0F990F\">%1</font></td>").arg(icount);
		
		   QString desc = query.value(3).toString();
		   s_Ret += QString("<td><font color=\"#0F990F\">%1</font></td></tr>").arg(desc);
		   
		   bfindet=true;
		}
	}
	
	if(!bfindet)
	   s_Ret += QString("<tr><td></td><td>%1</td><td></td></tr>").arg(QObject::tr("No transactions"));
	
	s_Ret += "</table>";
	    
	return s_Ret;
}


/*!
    \fn FDataBase::add_operation(bool bAdd, int count_d, const QString& date, const QString& descr)
 */
void add_operation(bool bAdd, Db_dat data)
{
	if ( !connectDB( "qfinances-db.db" ) )
		return;
	
	QSqlQuery query;
	
	query.prepare("select * from bank where 1");
	query.exec();
	int idata=0;
	while (query.next())
		idata = query.value(0).toInt();
	
	query.prepare("INSERT INTO operation (date_s, type_op, count, description) "
			"VALUES (:date_s, :type_op, :count, :description)");
	
	if(bAdd)
	{
		query.bindValue(":type_op", QObject::tr("+ Income"));
	   
		idata += data.count_d;
	}
	else
	{
		query.bindValue(":type_op", QObject::tr("- Outgoes"));
	   
		idata -= data.count_d;
	}
	
	if(idata<0)
	{
		QMessageBox::information(0,QObject::tr("Erorr operation"),QObject::tr("No cash"));
		return;	
	}
		
	query.bindValue(":date_s", data.date);
		
	query.bindValue(":count", data.count_d);
	query.bindValue(":description", data.descr);
	
	if ( !query.exec() )
		qDebug() << query.lastError();
	
	QString queryString = "delete from bank where 1";
	if (!query.exec(queryString))
	{
		qDebug() << "Failed to execute" << queryString;
		return;
	}
	
	query.prepare("INSERT INTO bank (count_c) "
			"VALUES (:count_c)");
	query.bindValue(":count_c", idata);
	
	if ( !query.exec() )
		qDebug() << query.lastError();
	
}


/*!
    \fn FDataBase::set_data_creditor(const QString& Name, int count_d, const QString& date, const QString& descr)
 */
void set_data_creditor(Db_dat data)
{
	if ( !connectDB( "qfinances-db.db" ) )
		return;
	
	QSqlQuery query;
	
	query.prepare("select * from bank where 1");
	query.exec();
	int idata=0;
	while (query.next())
		idata = query.value(0).toInt();
	
	
	query.prepare("select * from creditors where 1");
	query.exec();
	
	bool b=false;
	
	while (query.next())
	{
		QSqlQuery query_in;
		if(data.date == query.value(0).toString())
		{
			int n = query.value(2).toInt()-data.count_d;
			if(n > 0)
			{
			   query_in.prepare("update creditors set count=:count where (date_c=:date_c)");
			   query_in.bindValue(":count", n);
			   query_in.bindValue(":date_c", data.date);
			}
			else
			{
				query_in.prepare("delete from creditors where (date_c=:date_c)");
				query_in.bindValue(":date_c", data.date);
			}
			
			if ( !query_in.exec() )
			{
				qDebug() << query_in.lastError();
				return;
			}
			
			b=true;
			break;
		}
	}
	
	if(!b)
	{
		idata -= data.count_d;//надання кредиту і перевірка чи достатньо на рахунку грошей для здійснення операції
		if(idata<0)
		{
			QMessageBox::information(0,QObject::tr("Erorr operation"),QObject::tr("No cash"));
			return;	
		}
		
		
		query.prepare("INSERT INTO creditors (date_c, name_c, count, description) "
				"VALUES (:date_c, :name, :count, :description)");
		query.bindValue(":count", data.count_d);
		query.bindValue(":date_c", data.date);
		query.bindValue(":name", data.Name);
		query.bindValue(":description", data.descr);
		
		if ( !query.exec() )
		{
		   qDebug() << query.lastError();
		   return;
		}
	}
		
		
	query.prepare("INSERT INTO operation (date_s, type_op, count, description) "
			"VALUES (:date_s, :type_op, :count, :description)");
	
	if(b)
	{
		query.bindValue(":type_op", QObject::tr("+ Income"));
	   
		idata += data.count_d;
		query.bindValue(":description", QString("<b><i>Credit amortiazation</i></b><br><b>debtor:</b>")+data.Name);
	}
	else
	{
		query.bindValue(":type_op", QObject::tr("- Outgoes"));
		query.bindValue(":description", QString("<b><i>Add Credit</i></b><br><b>debtor:</b>")+data.Name+"<br><font color=\"blue\"><b>Description:</b>"+data.descr+"</font>");
	 
	}
		
		
	query.bindValue(":date_s", data.date);
		
	query.bindValue(":count", data.count_d);
		
	if ( !query.exec() )
	{
		qDebug() << query.lastError();
		return;
	}
	
	QString queryString = "delete from bank where 1";
	if (!query.exec(queryString))
	{
		qDebug() << "Failed to execute" << queryString;
		return;
	}
	
	query.prepare("INSERT INTO bank (count_c) "
			"VALUES (:count_c)");
	query.bindValue(":count_c", idata);
	
	if ( !query.exec() )
	{
		qDebug() << query.lastError();
		return;
	}
}


/*!
    \fn FDataBase::info_creditor(const QString& Name)
 */
Db_dat info_creditor(const QString& Name /*data set credit*/)
{
	Db_dat data;
	
	if ( !connectDB( "qfinances-db.db" ) )
		return data;
	
		
	QSqlQuery query;
	
	
	query.prepare("select * from creditors where 1");
	query.exec();
	
	bool b=false;
	
	while (query.next())
	{
		if(Name == query.value(0).toString())
		{
			data.Name = query.value(1).toString();
			data.descr = query.value(3).toString();
			data.date = query.value(0).toString();
			data.count_d = query.value(2).toInt();
			
			break;
		}	
	}
	
	return data;
}
