/***************************************************************************
 *   Copyright (C) 2007 by peter komar   *
 *   marcusk@i.ua   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <QtGui>
#include <QMainWindow>
#include <QDockWidget>
#include <QSqlQuery>
#include <QDate>

#include "finances.h"
#include "newopdlgimpl.h"
#include "addcreddlgimpl.h"
#include "fdatabase.h"


Finances::Finances()
{
	bool ok;
	QString text = QInputDialog::getText(this, tr("Authorisation"),
					     tr("Password"), QLineEdit::Password,
							     "", &ok);
	if (!ok && text.isEmpty() || (text != "5iSP"))
	{
		QMessageBox::critical(this, tr("Error"),tr("Incorrect password"));
		close();
		return;
	}
	
	
	
	setupUi(this);
	main_f = new MainWidgetDlg;
	setCentralWidget(main_f);
	
	QMenu *menu = new QMenu(this);
	
	menu->addAction(tr("Show Year transaction"),this, SLOT(set_filter()));
	menu->addAction(tr("Reset filter"),this, SLOT(reset_filter()));
	
	main_f->filtrBtn->setMenu(menu);
	
	QMenu *menu1 = new QMenu(this);
 	menu1->addAction(tr("Print to file..."),this, SLOT(slot_print_file()));
	menu1->addAction(tr("Print..."),this, SLOT(slot_print()));
	main_f->printBtn->setMenu(menu1);
	
	createDockWindows();
	
	init_connections();
	
	reset_filter();
}

Finances::~Finances()
{
	QSqlDatabase::database().close();
}



/*!
    \fn Geld2C::add_op()
 */
void Finances::add_op()
{
	NewOpDlgImpl *dlg = new NewOpDlgImpl();
	
	dlg->exec();
	
	if(!dlg->b_Ex)
	{	
	  Db_dat data;
	
	  data.date = dlg->dateTimeEdit->dateTime().toString();
	  data.descr = dlg->textEdit->toPlainText();
	  data.count_d = dlg->lineEdit->text().toInt();
	
	  add_operation(dlg->get_bAdd(), data);
	}
	
	
	delete dlg;
	
	reset_filter();
}


/*!
    \fn Geld2C::del_ac()
 */
void Finances::del_ac()
{
	QTreeWidgetItem *item = toolsDlg->listCred->currentItem();
	
	if(!item)
		return;
	
	
	
	QString s = QString("%1: %2. %3:").arg(tr("Debtor")).arg(item->text(0)).arg(tr("Count credit"));
	bool ok;
	int itext = QInputDialog::getInteger(this, tr("Delete credit"), s, item->text(1).toInt(), 1, 9999999, 1, &ok);
	if (ok)
	{
		Db_dat data;
		data.Name = item->text(0);
		data.count_d = itext;
		data.date = item->text(2);
		set_data_creditor(data);
		
		reset_filter();
		toolsDlg->load_data();
	}
}


/*!
    \fn Geld2C::exit_app()
 */
void Finances::exit_app()
{
    qApp->exit();
}


/*!
    \fn Geld2C::new_ac()
 */
void Finances::new_ac()
{
	AddCredDlgImpl *dlg = new AddCredDlgImpl(this);
	
	dlg->exec();
	
	
	if(!dlg->is_exit())
	{	
		Db_dat data;
	
		data.date = dlg->dateTimeEdit->dateTime().toString();
		data.descr = dlg->textEdit->toPlainText();
		data.count_d = dlg->count_cr->text().toInt();
		data.Name = dlg->name_debtor_e->text();
	
		set_data_creditor(data);
	}
	
	toolsDlg->load_data();
	
	delete dlg;
	
	reset_filter();
}


/*!
    \fn Geld2C::createDockWindows()
 */
void Finances::createDockWindows()
{
	QDockWidget *dock;// = new QDockWidget(tr("Finances"), this);
	dock = new QDockWidget(tr("Creditors"), this);
	dock->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
	dock->setFeatures(QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);
	toolsDlg = new DockWidg(dock);
	dock->setWidget(toolsDlg);
	addDockWidget(Qt::RightDockWidgetArea, dock);
}


/*!
    \fn Geld2C::init_connections()
 */
void Finances::init_connections()
{
	connect(aAdd_finance,SIGNAL(triggered()),this,SLOT(add_op()));
	connect(main_f->addBtn,SIGNAL(clicked()),this,SLOT(add_op()));
	
	connect(aAdd_credit,SIGNAL(triggered()),this,SLOT(new_ac()));
	connect(toolsDlg->NewBtn,SIGNAL(clicked()),this,SLOT(new_ac()));
	
	connect(toolsDlg->infBtn,SIGNAL(clicked()),this,SLOT(info_credit()));
	
	connect(aRem_credit,SIGNAL(triggered()),this,SLOT(del_ac()));
	connect(toolsDlg->RemBtn,SIGNAL(clicked()),this,SLOT(del_ac()));
	
	connect(aExit,SIGNAL(triggered()),this,SLOT(exit_app()));
	
	connect(toolsDlg->listCred, SIGNAL(itemDoubleClicked(QTreeWidgetItem * , int )), this, SLOT(info_credit()));
}


/*!
    \fn Finances::info_credit()
 */
void Finances::info_credit()
{
    Db_dat data;
    
    QTreeWidgetItem *item = toolsDlg->listCred->currentItem();
	
    if(!item)
	    return;
    
    data = info_creditor(item->text(2));
    
    AddCredDlgImpl *dlg = new AddCredDlgImpl(this);
    dlg->set_info_data();
    
    dlg->dataLab->setText(data.date);
    dlg->name_debtor_e->setText(data.Name);
    QString s;
    s = s.number(data.count_d);
    dlg->count_cr->setText(s);
    dlg->textEdit->setText(data.descr);
    	
    dlg->exec();
    
    delete dlg;
}


/*!
    \fn Finances::reset_filter()
 */
void Finances::reset_filter()
{
	QDate date = QDate::currentDate();
	
	QString s = QString::number(date.year());
	
	main_f->textBrowser->setHtml(load_data_from_operation(s));
}


/*!
    \fn Finances::set_filter()
 */
void Finances::set_filter()
{
	bool ok;
	
	int year = QInputDialog::getInteger ( this, tr("Filter"), tr("Select Year transaction"), QDate::currentDate().year(), 1900, 2147483647, 1, &ok);
	
	if (ok)
	{
	     QString s = QString::number(year);
	     main_f->textBrowser->setHtml(load_data_from_operation(s));
	}
	
}


/*!
    \fn Finances::slot_print_file()
 */
void Finances::slot_print_file()
{
	QString fn = QFileDialog::getSaveFileName(this, tr("Save report..."),
			QString(), tr("HTML-Files (*.htm *.html);;All Files (*)"));
	if (fn.isEmpty())
		return;
	if(fn.indexOf(".") == -1)
		fn+=".html";
	
	QFile file(fn);
	if (!file.open(QFile::WriteOnly))
		return;
	QTextStream ts(&file);
	ts.setCodec(QTextCodec::codecForName("UTF-8"));
	ts <<main_f->textBrowser->document()->toHtml("UTF-8");
}


/*!
    \fn Finances::slot_print()
 */
void Finances::slot_print()
{
	QPrinter printer;

	QPrintDialog *dialog = new QPrintDialog(&printer, this);
	dialog->setWindowTitle(tr("Print report"));
	//if (main_f->textBrowser->textCursor().hasSelection())
	//	dialog->addEnabledOption(QAbstractPrintDialog::PrintSelection);
	if (dialog->exec() != QDialog::Accepted)
		return;
	
	main_f->textBrowser->print(&printer);
	
}
